import numpy as np

from openquake.hazardlib.imt import PGA, PGV, SA
from gmice.gmice import GMICE


class AK07(GMICE):
    """
    Implements the ground motion intensity conversion equations (GMICE) of
    Atkinson and Kaka (2007). This module implements a simplified version.

    References:
        Atkinson, Gail & Kaka, SanLinn. (2007). Relationships between
        Felt Intensity and Instrumental Ground Motion in the Central
        United States and California. Bulletin of The Seismological
        Society of America - BULL SEISMOL SOC AMER.
        97. 497-510. 10.1785/0120060154.
    """

    # -----------------------------------------------------------------------
    #
    # MMI = C1 + C2 * log(Y)          for c2->T1 < log(Y) <= T1
    # MMI = C3 + C4 * log(Y)          for log(Y) > T1
    #
    # or
    #
    # MMI = c2->C1 + c2->C2 * log(Y) + C5 + C6 * M + C7 * log(D)
    #                            for log(Y) <= c2->T1
    # MMI = C1 + C2 * log(Y) + C5 + C6 * log(D) + C7 * M
    #                            for c2->T1 < log(Y) <= T1
    # MMI = C3 + C4 * log(Y) + C5 + C6 * M + C7 * log(D)
    #                           for log(Y) > T1
    #
    # Limit the distance residuals to between 10 and 300 km.
    # Limit the magnitude residuals to between M3.0 and M7.3.
    #
    # Constants taken from Table 4 and 5 in the reference material
    #
    # -----------------------------------------------------------------------
    def __init__(self):
        super().__init__()
        self.min_max = (1.0, 10.0)
        self.name = "Atkinson and Kaka (2007)"
        self.scale = "scale_ak07.ps"
        self._constants = {
            self._pga: {
                "C1": 2.65,
                "C2": 1.39,
                "C3": -1.91,
                "C4": 4.09,
                "C5": -1.96,
                "C6": 0.02,
                "C7": 0.98,
                "T1": 1.69,
                "T2": 5,
                "SMMI": 0.89,
            },
            self._pgv: {
                "C1": 4.37,
                "C2": 1.32,
                "C3": 3.54,
                "C4": 3.03,
                "C5": 0.47,
                "C6": -0.19,
                "C7": 0.26,
                "T1": 0.48,
                "T2": 5,
                "SMMI": 0.76,
            },
            self._sa03: {
                "C1": 2.4,
                "C2": 1.36,
                "C3": -1.83,
                "C4": 3.56,
                "C5": -0.11,
                "C6": -0.2,
                "C7": 0.64,
                "T1": 1.92,
                "T2": 5,
                "SMMI": 0.79,
            },
            self._sa10: {
                "C1": 3.23,
                "C2": 1.18,
                "C3": 0.57,
                "C4": 2.95,
                "C5": 1.92,
                "C6": -0.39,
                "C7": 0.04,
                "T1": 1.5,
                "T2": 5,
                "SMMI": 0.73,
            },
            self._sa30: {
                "C1": 3.72,
                "C2": 1.29,
                "C3": 1.99,
                "C4": 3.0,
                "C5": 2.24,
                "C6": -0.33,
                "C7": -0.31,
                "T1": 1,
                "T2": 5,
                "SMMI": 0.72,
            },
        }

        self.DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([PGA, PGV, SA])

        self.DEFINED_FOR_SA_PERIODS = set([0.3, 1.0, 3.0])

    def getMIfromGM(self, amps, imt, dists=None, mag=None):
        """
        Function to compute macroseismic intensity from ground-motion
        intensity. Supported ground-motion IMTs are PGA, PGV and PSA
        at 0.3, 1.0, and 2.0 sec periods.

        Args:
            amps (ndarray):
                Ground motion amplitude; natural log units; g for PGA and
                PSA, cm/s for PGV.
            imt (OpenQuake IMT):
                Type the input amps (must be one of PGA, PGV, or SA).
                Supported SA periods are 0.3, 1.0, and 3.0 sec.
                `[link] <http://docs.openquake.org/oq-hazardlib/master/imt.html>`
            dists (ndarray):
                Numpy array of distances from rupture (km).
            mag (float):
                Earthquake magnitude.

        Returns:
            ndarray of Modified Mercalli Intensity and ndarray of
            dMMI / dln(amp) (i.e., the slope of the relationship at the
            point in question).
        """
        lfact = np.log10(np.e)
        c = self._getConsts(imt)
        if dists is not None and mag is not None:
            doresid = True
            # Limit distances and take the log10
            ldd = np.log10(np.clip(dists, 10, 300))
            # Llimit magnitudes
            lmm = np.clip(mag, 3.0, 7.3)
        else:
            doresid = False

        # Account for ln form of amplitudes
        # Convert (for accelerations) from ln(g) to cm/s^2
        # then take the log10
        #
        if imt != self._pgv:
            units = 981.0
        else:
            units = 1.0
        #
        # Math: log10(981 * exp(amps)) = log10(981) + log10(exp(amps))
        # = log10(981) + amps * log10(e)
        # For PGV, just convert ln(amp) to log10(amp) by multiplying
        # by log10(e)
        #
        lamps = np.log10(units) + amps * lfact
        mmi = np.zeros_like(amps)
        dmmi_damp = np.zeros_like(amps)
        #
        # This is the lower segment of the bi-linear fit
        # where log(Y) is less than T1
        #
        idx = lamps < c["T1"]
        mmi[idx] = c["C1"] + c["C2"] * lamps[idx]
        dmmi_damp[idx] = c["C2"] * lfact
        #
        # This is the upper segment of the bi-linear fit
        # where log(Y) is greater than or equal to T1
        #
        idx = lamps >= c["T1"]
        mmi[idx] = c["C3"] + c["C4"] * lamps[idx]
        dmmi_damp[idx] = c["C4"] * lfact

        # Inclusion of residuals if magnitude and
        # distance information is available
        if doresid:
            mmi += c["C5"] + c["C6"] * lmm + c["C7"] * ldd

        # Limit mmi values
        mmi = np.clip(mmi, 1.0, 10.0)
        return mmi, dmmi_damp

    def getGMfromMI(self, mmi, imt, dists=None, mag=None):
        """
        Function to tcompute ground-motion intensity from macroseismic
        intensity. Supported IMTs are PGA, PGV and PSA for 0.3, 1.0, and
        3.0 sec periods.

        Args:
            mmi (ndarray):
                Macroseismic intensity.
            imt (OpenQuake IMT):
                IMT of the requested ground-motions intensities (must be
                one of PGA, PGV, or SA).
                `[link] <http://docs.openquake.org/oq-hazardlib/master/imt.html>`
            dists (ndarray):
                Rupture distances (km) to the corresponding MMIs.
            mag (float):
                Earthquake magnitude.

        Returns:
            Ndarray of ground motion intensity in natural log of g for PGA
            and PSA, and natural log cm/s for PGV; ndarray of dln(amp) / dMMI
            (i.e., the slope of the relationship at the point in question).
        """
        lfact = np.log10(np.e)
        c = self._getConsts(imt)
        mmi = mmi.copy()
        # Set nan values to 1
        ix_nan = np.isnan(mmi)
        mmi[ix_nan] = 1.0

        if dists is not None and mag is not None:
            doresid = True
            # Limit distances and take the log10
            ldd = np.log10(np.clip(dists, 10, 300))
            # Limit magnitudes
            lmm = np.clip(mag, 3.0, 7.3)
        else:
            doresid = False

        # Inclusion of residuals if magnitude and
        # distance information is available
        if doresid:
            mmi -= c["C5"] + c["C6"] * lmm + c["C7"] * ldd

        pgm = np.zeros_like(mmi)
        dpgm_dmmi = np.zeros_like(mmi)
        #
        # This is the lower segment of the bi-linear fit
        # where MMI is less than I5
        #
        idx = mmi < c["T2"]
        pgm[idx] = np.power(10, (mmi[idx] - c["C1"]) / c["C2"])
        dpgm_dmmi[idx] = 1.0 / (c["C2"] * lfact)
        #
        # This is the upper segment of the bi-linear fit
        # where MMI is greater than or equal to I5
        #
        idx = mmi >= c["T2"]
        pgm[idx] = np.power(10, (mmi[idx] - c["C3"]) / c["C4"])
        dpgm_dmmi[idx] = 1.0 / (c["C4"] * lfact)

        if imt != self._pgv:
            units = 981.0
        else:
            units = 1.0

        # Return a ln(amp) value. Convert PGA to from cm/s^2 to g
        pgm /= units
        pgm = np.log(pgm)

        # Set nan values back from 1 to nan
        pgm[ix_nan] = np.nan
        dpgm_dmmi[ix_nan] = np.nan

        return pgm, dpgm_dmmi

    def getGM2MIsd(self):
        """
        Return a dictionary of standard deviations for the ground-motion
        to MMI conversion. The keys are the ground motion types.

        Returns:
            Dictionary of GM to MI sigmas (in MMI units).
        """
        return {
            self._pga: self._constants[self._pga]["SMMI"],
            self._pgv: self._constants[self._pgv]["SMMI"],
            self._sa03: self._constants[self._sa03]["SMMI"],
            self._sa10: self._constants[self._sa10]["SMMI"],
            self._sa30: self._constants[self._sa30]["SMMI"],
        }

    def getMI2GMsd(self):
        """
        Return a dictionary of standard deviations for the MMI
        to ground-motion conversion. The keys are the ground motion
        types.

        Returns:
            Dictionary of MI to GM sigmas (ln(PGM) units).
        """
        #
        # Need to convert log10 to ln units
        #
        lfact = np.log(10.0)
        return {
            self._pga: lfact * 0.57,
            self._pgv: lfact * 0.52,
            self._sa03: lfact * 0.63,
            self._sa10: lfact * 0.57,
            self._sa30: lfact * 0.81,
        }

    def _getConsts(self, imt):
        """
        Helper function to get the constants.
        """

        if (
            imt != self._pga
            and imt != self._pgv
            and imt != self._sa03
            and imt != self._sa10
            and imt != self._sa30
        ):
            raise ValueError("Invalid IMT " + str(imt))
        c = self._constants[imt]
        return c



# third party imports
import numpy as np

# stdlib imports
from openquake.hazardlib import imt
from gmice.gmice import GMICE


class Wald99(GMICE):
    """
    Implements the ground motion intensity conversion equations (GMICE) of
    Wald et al. (1999). This module implements a simplified version in that
    it only uses one of PGV or PGA, and not a combination of the two (PGV
    for higher intensities, PGA for lower) as is recommended in the reference.

    References:
        Wald, D.J., V. Quitoriano, T.H. Heaton, and H. Kanamori (1999).
        Relationships between peak gorund acceleration, peak ground
        velocity, and Modified Mercalli Intensity in California,
        Earthquake Spectra, Volume15, No. 3, August 1999.
    """

    # -----------------------------------------------------------------------
    #
    # Imm = 3.66 * log10(PGA) - 1.66 for Imm >= V (e.g., log10(PGA) >= 1.82)
    # Imm = 2.20 * log10(PGA) + 1.0  for Imm < 5
    #
    # Imm = 3.47 * log10(PGV) + 2.35 for Imm >= V
    # Imm = 2.10 * log10(PGV) + 3.40  for Imm < 5
    #
    # -----------------------------------------------------------------------
    def __init__(self):
        super().__init__()
        self.min_max = (1.0, 10.0)
        self.name = "Wald et al. (1999)"
        self.scale = "scale_wald99.ps"
        self.__constants = {
            self._pga: {
                "C1": 3.66,
                "C2": -1.66,
                "C3": 2.20,
                "C4": 1.00,
                "T1": 1.82,
                "T2": 5.00,
                "SMMI": 1.08,
                "SPGM": 0.295,
            },
            self._pgv: {
                "C1": 3.47,
                "C2": 2.35,
                "C3": 2.10,
                "C4": 3.40,
                "T1": 0.76,
                "T2": 5.00,
                "SMMI": 0.98,
                "SPGM": 0.282,
            },
        }

        self.DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([imt.PGA, imt.PGV])

        self.DEFINED_FOR_SA_PERIODS = set([])

    def getPreferredMI(self, df, dists=None, mag=None):
        if "PGA" not in df or "PGV" not in df:
            if "PGV" in df:
                oqimt = imt.from_string("PGV")
                return self.getMIfromGM(df["PGV"], oqimt, dists, mag)[0]
            elif "PGA" in df:
                oqimt = imt.from_string("PGA")
                return self.getMIfromGM(df["PGA"], oqimt, dists, mag)[0]
            else:
                return None
        oqimt = imt.from_string("PGA")
        mmi_pga = self.getMIfromGM(df["PGA"], oqimt, dists, mag)[0]
        ix_nan_pga = np.isnan(mmi_pga)
        oqimt = imt.from_string("PGV")
        mmi_pgv = self.getMIfromGM(df["PGV"], oqimt, dists, mag)[0]
        ix_nan_pgv = np.isnan(mmi_pgv)
        ix_nan = ix_nan_pga | ix_nan_pgv
        vscale = np.zeros_like(mmi_pga)
        vscale[~ix_nan] = (mmi_pga[~ix_nan] - 5) / 2
        vscale[vscale < 0] = 0
        vscale[vscale > 1] = 1
        ascale = 1 - vscale
        mmi = np.full_like(mmi_pga, np.nan)
        mmi[~ix_nan] = np.clip(
            mmi_pga[~ix_nan] * ascale[~ix_nan] + mmi_pgv[~ix_nan] * vscale[~ix_nan],
            1,
            10,
        )
        mmi[ix_nan_pgv] = mmi_pga[ix_nan_pgv]
        mmi[ix_nan_pga] = mmi_pgv[ix_nan_pga]
        ix_nan = np.isnan(mmi)
        mmi95 = np.full_like(mmi, False, dtype=bool)
        mmi95[~ix_nan] = mmi[~ix_nan] > 9.5
        mmi[mmi95] = 10.0
        return mmi

    def getMIfromGM(self, amps, imt, dists=None, mag=None):
        """
        Function to compute macroseismic intensity from ground-motion
        intensity. Supported ground-motion IMTs are PGA and PGV

        Args:
            amps (ndarray):
                Ground motion amplitude; natural log units; g for PGA and
                PSA, cm/s for PGV.
            imt (OpenQuake IMT):
                Type the input amps (must be one of PGA or PGV).
                `[link] <http://docs.openquake.org/oq-hazardlib/master/imt.html>`
            dists (ndarray):
                Numpy array of distances from rupture (km). This parameter
                is ignored by this GMICE.
            mag (float):
                Earthquake magnitude. This parameter is ignored by this
                GMICE.

        Returns:
            ndarray of Modified Mercalli Intensity and ndarray of
            dMMI / dln(amp) (i.e., the slope of the relationship at the
            point in question).
        """  # noqa
        lfact = np.log10(np.e)
        c = self._getConsts(imt)
        ix_nan = np.isnan(amps)

        #
        # Convert (for accelerations) from ln(g) to cm/s^2
        # then take the log10
        #
        if imt == self._pga:
            units = 981.0
        else:
            units = 1.0
        #
        # Math: log10(981 * exp(amps)) = log10(981) + log10(exp(amps))
        # = log10(981) + amps * log10(e)
        # For PGV, just convert ln(amp) to log10(amp) by multiplying
        # by log10(e)
        #
        lamps = np.zeros_like(amps)
        lamps[~ix_nan] = np.log10(units) + amps[~ix_nan] * lfact
        mmi = np.full_like(amps, np.nan)
        dmmi_damp = np.zeros_like(amps)
        #
        # This is the upper segment of the bi-linear fit
        #
        idx = lamps >= c["T1"]
        mmi[idx] = c["C2"] + c["C1"] * lamps[idx]
        dmmi_damp[idx] = c["C1"] * lfact
        #
        # This is the lower segment of the bi-linear fit
        #
        idx = lamps < c["T1"]
        mmi[idx] = c["C4"] + c["C3"] * lamps[idx]
        dmmi_damp[idx] = c["C3"] * lfact

        mmi = np.clip(mmi, 1.0, 10.0)
        mmi[ix_nan] = np.nan
        return mmi, dmmi_damp

    def getGMfromMI(self, mmi, imt, dists=None, mag=None):
        """
        Function to tcompute ground-motion intensity from macroseismic
        intensity. Supported IMTs are PGA and PGV.

        Args:
            mmi (ndarray):
                Macroseismic intensity.
            imt (OpenQuake IMT):
                IMT of the requested ground-motions intensities (must be
                one of PGA or PGV).
                `[link] <http://docs.openquake.org/oq-hazardlib/master/imt.html>`
            dists (ndarray):
                Rupture distances (km) to the corresponding MMIs. This
                parameter is ignored by this GMICE.
            mag (float):
                Earthquake magnitude. This parameter is ignored by this
                GMICE.

        Returns:
            Ndarray of ground motion intensity in natural log of g for PGA
            and PSA, and natural log cm/s for PGV; ndarray of dln(amp) / dMMI
            (i.e., the slope of the relationship at the point in question).
        """  # noqa
        lfact = np.log10(np.e)
        c = self._getConsts(imt)
        mmi = mmi.copy()
        ix_nan = np.isnan(mmi)
        mmi[ix_nan] = 1.0

        pgm = np.zeros_like(mmi)
        dpgm_dmmi = np.zeros_like(mmi)

        #
        # Upper segment of bi-linear relationship
        #
        idx = mmi >= c["T2"]
        pgm[idx] = np.power(10, (mmi[idx] - c["C2"]) / c["C1"])
        dpgm_dmmi[idx] = 1.0 / (c["C1"] * lfact)
        #
        # Lower segment of bi-linear relationship
        #
        idx = mmi < c["T2"]
        pgm[idx] = np.power(10, (mmi[idx] - c["C4"]) / c["C3"])
        dpgm_dmmi[idx] = 1.0 / (c["C3"] * lfact)

        if imt != self._pgv:
            units = 981.0
        else:
            units = 1.0
        pgm /= units
        pgm = np.log(pgm)
        pgm[ix_nan] = np.nan
        dpgm_dmmi[ix_nan] = np.nan

        return pgm, dpgm_dmmi

    def getGM2MIsd(self):
        """
        Return a dictionary of standard deviations for the ground-motion
        to MMI conversion. The keys are the ground motion types.

        Returns:
            Dictionary of GM to MI sigmas (in MMI units).
        """
        return {
            self._pga: self.__constants[self._pga]["SMMI"],
            self._pgv: self.__constants[self._pgv]["SMMI"],
        }

    def getMI2GMsd(self):
        """
        Return a dictionary of standard deviations for the MMI
        to ground-motion conversion. The keys are the ground motion
        types.

        Returns:
            Dictionary of MI to GM sigmas (ln(PGM) units).
        """
        #
        # Need to convert log10 to ln units
        #
        lfact = np.log(10.0)
        return {
            self._pga: lfact * self.__constants[self._pga]["SPGM"],
            self._pgv: lfact * self.__constants[self._pgv]["SPGM"],
        }

    def _getConsts(self, imt):
        """
        Helper function to get the constants.
        """

        if imt != self._pga and imt != self._pgv:
            raise ValueError("Invalid IMT " + str(imt))
        return self.__constants[imt]



import numpy as np

from openquake.hazardlib.imt import PGA, PGV, SA
from gmice.gmice import GMICE


class FM11(GMICE):
    """
    Implements the ground motion intensity conversion equations (GMICE) of
    Faenza and Michelini (2010, 2011).

    References:
        Faenza and Michelini,(2010). Regression analysis of MCS intensity and
        ground motion parameters in Italy and its application in ShakeMap.
        GJI, 180, 1138-1152, doi: 10.1111/j.1365-246X.2009.04467.xself
        and Faenza and Michelini (2011). Regression analysis of MCS intensity
        and ground motion spectral accelerations (SAs) in Italy. GJI, 186,
        1415-1430, doi: 10.1111/j.1365-246X.2011.05125.x
    """

    # -----------------------------------------------------------------------
    # MMI = C1 + C2 * log10 (Y)
    #
    # Limit the distance residuals to between 0 and 160 km.
    # Limit the magnitude residuals to between M3.0 and M6.9.
    # These are the default in the other gmice
    #      Limit the distance residuals to between 10 and 300 km.
    #      Limit the magnitude residuals to between M3.0 and M7.3.
    #
    # These are calcualted on the basis of the maximum horizontal component.
    # For psa 03, 10 and 30 the regression for the geometrical mean are
    # available but not implemented in this modeles since the one for PGA
    # and PGV are not available.
    # -----------------------------------------------------------------------
    def __init__(self):
        super().__init__()
        self.min_max = (1.0, 10.0)
        self.name = "Faenza and Michelini (2010, 2011)"
        self.scale = "scale_fm11.ps"
        self._constants = {
            self._pga: {"C1": 1.68, "C2": 2.58, "SMMI": 0.18, "SPGM": 0.31},
            self._pgv: {"C1": 5.11, "C2": 2.35, "SMMI": 0.14, "SPGM": 0.22},
            self._sa03: {"C1": 1.24, "C2": 2.47, "SMMI": 0.30, "SPGM": 0.42},
            self._sa10: {"C1": 3.12, "C2": 2.05, "SMMI": 0.21, "SPGM": 0.31},
            self._sa30: {"C1": 4.31, "C2": 2.00, "SMMI": 0.14, "SPGM": 0.26},
        }

        self.DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([PGA, PGV, SA])

        self.DEFINED_FOR_SA_PERIODS = set([0.3, 1.0, 3.0])

    def getMIfromGM(self, amps, imt, dists=None, mag=None):
        """
        Function to compute macroseismic intensity from ground-motion
        intensity. Supported ground-motion IMTs are PGA, PGV and PSA
        at 0.3, 1.0, and 3.0 sec periods.

        Args:
            amps (ndarray):
                Ground motion amplitude; natural log units; g for PGA and
                PSA, cm/s for PGV.
            imt (OpenQuake IMT):
                Type the input amps (must be one of PGA, PGV, or SA).
                Supported SA periods are 0.3, 1.0, and 3.0 sec.
                `[link] <http://docs.openquake.org/oq-hazardlib/master/imt.html>`
            dists (ndarray):
                Not used
            mag (float):
                Not used

        Returns:
            ndarray of Modified Mercalli Intensity and ndarray of
            dMMI / dln(amp) (i.e., the slope of the relationship at the
            point in question).
        """  # noqa
        lfact = np.log10(np.e)
        c = self._getConsts(imt)

        #
        # Convert (for accelerations) from ln(g) to cm/s^2
        # then take the log10
        #
        if imt != self._pgv:
            units = 981.0
        else:
            units = 1.0
        #
        # Math: log10(981 * exp(amps)) = log10(981) + log10(exp(amps))
        # = log10(981) + amps * log10(e)
        # For PGV, just convert ln(amp) to log10(amp) by multiplying
        # by log10(e)
        #
        lamps = np.log10(units) + amps * lfact

        mmi = c["C1"] + c["C2"] * lamps
        dmmi_damp = np.full_like(lamps, c["C2"] * lfact)

        mmi = np.clip(mmi, 1.0, 10.0)
        mmi[np.isnan(amps)] = np.nan
        return mmi, dmmi_damp

    def getGMfromMI(self, mmi, imt, dists=None, mag=None):
        """
        Function to tcompute ground-motion intensity from macroseismic
        intensity. Supported IMTs are PGA, PGV and PSA for 0.3, 1.0, and
        3.0 sec periods.

        Args:
            mmi (ndarray):
                Macroseismic intensity.
            imt (OpenQuake IMT):
                IMT of the requested ground-motions intensities (must be
                one of PGA, PGV, or SA).
                `[link] <http://docs.openquake.org/oq-hazardlib/master/imt.html>`
            dists (ndarray):
                Not used
            mag (float):
                Not used

        Returns:
            Ndarray of ground motion intensity in natural log of g for PGA
            and PSA, and natural log cm/s for PGV; ndarray of dln(amp) / dMMI
            (i.e., the slope of the relationship at the point in question).
        """  # noqa
        lfact = np.log10(np.e)
        c = self._getConsts(imt)
        mmi = mmi.copy()
        # Set nan values to 1
        ix_nan = np.isnan(mmi)
        mmi[ix_nan] = 1.0

        pgm = np.zeros_like(mmi)
        dpgm_dmmi = np.zeros_like(mmi)
        dummy_variable = np.ones(len(mmi))

        #
        # MMI to PGM
        #
        pgm = np.power(10, (mmi - c["C1"]) / c["C2"])
        dpgm_dmmi = 1.0 / (c["C2"] * lfact) * dummy_variable

        if imt != self._pgv:
            units = 981.0
        else:
            units = 1.0

        # Return a ln(amp) value. Convert PGA to from cm/s^2 to g
        pgm /= units
        pgm = np.log(pgm)

        # Set nan values back from 1 to nan
        pgm[ix_nan] = np.nan
        dpgm_dmmi[ix_nan] = np.nan

        return pgm, dpgm_dmmi

    def getGM2MIsd(self):
        """
        Return a dictionary of standard deviations for the ground-motion
        to MMI conversion. The keys are the ground motion types.

        Returns:
            Dictionary of GM to MI sigmas (in MMI units).
        """
        return {
            self._pga: self._constants[self._pga]["SMMI"],
            self._pgv: self._constants[self._pgv]["SMMI"],
            self._sa03: self._constants[self._sa03]["SMMI"],
            self._sa10: self._constants[self._sa10]["SMMI"],
            self._sa30: self._constants[self._sa30]["SMMI"],
        }

    def getMI2GMsd(self):
        """
        Return a dictionary of standard deviations for the MMI
        to ground-motion conversion. The keys are the ground motion
        types.

        Returns:
            Dictionary of MI to GM sigmas (ln(PGM) units).
        """
        #
        # Need to convert log10 to ln units
        #
        lfact = np.log(10.0)
        return {
            self._pga: lfact * self._constants[self._pga]["SPGM"],
            self._pgv: lfact * self._constants[self._pgv]["SPGM"],
            self._sa03: lfact * self._constants[self._sa03]["SPGM"],
            self._sa10: lfact * self._constants[self._sa10]["SPGM"],
            self._sa30: lfact * self._constants[self._sa30]["SPGM"],
        }

    def _getConsts(self, imt):
        """
        Helper function to get the constants.
        """

        if (
            imt != self._pga
            and imt != self._pgv
            and imt != self._sa03
            and imt != self._sa10
            and imt != self._sa30
        ):
            raise ValueError("Invalid IMT " + str(imt))
        c = self._constants[imt]
        return c



# stdlib imports
from abc import ABC, abstractmethod

from openquake.hazardlib import imt


class GMICE(ABC):
    """
    Base class called to implement ground motion intensity conversion
    equations (GMICE).

    Inherited by AK07, Wald99, WGRW12.
    """

    def __init__(self):
        self._pga = imt.PGA()
        self._pgv = imt.PGV()
        self._sa03 = imt.SA(0.3)
        self._sa10 = imt.SA(1.0)
        self._sa30 = imt.SA(3.0)
        self.DEFINED_FOR_INTENSITY_MEASURE_TYPES = set()

    @staticmethod
    def getDistanceType():
        return "rrup"

    def supports(self, imt_str):
        """Determine whether input IMT is supported by GMICE instance.

        Args:
            imt_str (str): Valid IMT string - 'MMI', 'PGV', 'PGA', 'SA(0.3)', etc.
        Returns:
            bool: True if gmice is defined for input IMT (and period), False
                  if not.
        """
        for imtcomp in self.DEFINED_FOR_INTENSITY_MEASURE_TYPES:
            thisimt = imt.from_string(imt_str)
            if thisimt.string == imtcomp.__name__:
                return True
            if "SA" in thisimt.string:
                for period in self.DEFINED_FOR_SA_PERIODS:
                    if period == thisimt.period:
                        return True
                return False
        return False

    def getMinMax(self):
        """
        Get the minimum and maximum MMI values produced by this GMICE.

        Returns:
            Tuple of min and max values of GMICE.
        """
        return self.min_max

    def getName(self):
        """
        Get the name of this GMICE.

        Returns:
            String containing name of GMICE.
        """
        return self.name

    def getScale(self):
        """
        Get the name of the PostScript file containing this GMICE's
        scale.

        Returns:
            Name of GMICE scale file.
        """
        return self.scale

    def getPreferredMI(self, df, dists=None, mag=None):
        """
        Function to compute macroseismic intensity from the preferred
        ground-motion intensity. The function uses PGV by default, but
        this may be overridden by individual classes.

        Args:
            df (dict):
                Dictionaary containing all of the available ground
                motions.
            dists (ndarray):
                Numpy array of distances from rupture (km).
            mag (float):
                Earthquake magnitude.

        Returns:
            ndarray of Modified Mercalli Intensity and ndarray of
            dMMI / dln(amp) (i.e., the slope of the relationship at the
            point in question).
        """
        if "PGV" not in df:
            return None
        oqimt = imt.from_string("PGV")
        return self.getMIfromGM(df["PGV"], oqimt, dists, mag)[0]

    def getPreferredSD(self):
        """
        Return an array of standard deviations for the preferred
        ground-motion to MMI conversion. Return None is the preferred
        IMT is not in the list of available IMTs for the GMICE.

        Returns:
            (numpy array):  Array of GM to MI sigmas (in MMI units).
        """
        oqimt = imt.from_string("PGV")
        sd = self.getGM2MIsd()
        if oqimt in sd:
            return sd[oqimt]
        else:
            return None

    @abstractmethod
    def getMIfromGM(self, amps, imt, dists=None, mag=None):
        """
        Function to compute macroseismic intensity from ground-motion
        intensity. Supported ground-motion IMTs are PGA, PGV and PSA
        at 0.3, 1.0, and 2.0 sec periods.

        Args:
            amps (ndarray):
                Ground motion amplitude; natural log units; g for PGA and
                PSA, cm/s for PGV.
            imt (OpenQuake IMT):
                Type the input amps (must be one of PGA, PGV, or SA).
                Supported SA periods are 0.3, 1.0, and 3.0 sec.
                `[link] <http://docs.openquake.org/oq-hazardlib/master/imt.html>`
            dists (ndarray):
                Numpy array of distances from rupture (km).
            mag (float):
                Earthquake magnitude.

        Returns:
            ndarray of Modified Mercalli Intensity and ndarray of
            dMMI / dln(amp) (i.e., the slope of the relationship at the
            point in question).
        """  # noqa
        pass

    @abstractmethod
    def getGMfromMI(self, mmi, imt, dists=None, mag=None):
        """
        Function to tcompute ground-motion intensity from macroseismic
        intensity. Supported IMTs are PGA, PGV and PSA for 0.3, 1.0, and
        3.0 sec periods.

        Args:
            mmi (ndarray):
                Macroseismic intensity.
            imt (OpenQuake IMT):
                IMT of the requested ground-motions intensities (must be
                one of PGA, PGV, or SA).
                `[link] <http://docs.openquake.org/oq-hazardlib/master/imt.html>`
            dists (ndarray):
                Rupture distances (km) to the corresponding MMIs.
            mag (float):
                Earthquake magnitude.

        Returns:
            Ndarray of ground motion intensity in natural log of g for PGA
            and PSA, and natural log cm/s for PGV; ndarray of dln(amp) / dMMI
            (i.e., the slope of the relationship at the point in question).
        """  # noqa
        pass

    @abstractmethod
    def getGM2MIsd(self):
        """
        Return a dictionary of standard deviations for the ground-motion
        to MMI conversion. The keys are the ground motion types.

        Returns:
            Dictionary of GM to MI sigmas (in MMI units).
        """
        pass

    @abstractmethod
    def getMI2GMsd(self):
        """
        Return a dictionary of standard deviations for the MMI
        to ground-motion conversion. The keys are the ground motion
        types.

        Returns:
            Dictionary of MI to GM sigmas (ln(PGM) units).
        """
        pass

    @abstractmethod
    def _getConsts(self, imt):
        """
        Helper function to get the constants.
        """
        pass
